package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.SysMusicCompareRecordDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.SysMusicCompareRecord;
import com.ym.mec.biz.dal.enums.FeatureType;
import com.ym.mec.biz.dal.page.SysMusicCompareRecordQueryInfo;
import com.ym.mec.biz.service.SysMusicCompareRecordService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SysMusicCompareRecordServiceImpl extends BaseServiceImpl<Long, SysMusicCompareRecord>  implements SysMusicCompareRecordService {
	
	@Autowired
	private SysMusicCompareRecordDao sysMusicCompareRecordDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private StudentDao studentDao;

	@Override
	public BaseDAO<Long, SysMusicCompareRecord> getDAO() {
		return sysMusicCompareRecordDao;
	}

	@Override
	public long insert(SysMusicCompareRecord bean) {
		studentDao.addStudentCloudStudySequenceDays(bean.getUserId());
		return super.insert(bean);
	}

	@Override
	public void saveMusicCompareData(String phone, SoundCompareHelper soundCompareInfo) {
		SysUser user = teacherDao.getUserWithPhone(phone);
		if(Objects.isNull(user)){
			return;
		}
		SysMusicCompareRecord sysMusicCompareRecord = new SysMusicCompareRecord();
		sysMusicCompareRecord.setUserId(user.getId());
		sysMusicCompareRecord.setSysMusicScoreId(soundCompareInfo.getMusicScoreId());
		sysMusicCompareRecord.setBehaviorId(soundCompareInfo.getBehaviorId());
		sysMusicCompareRecord.setScoreData(JSON.toJSONString(soundCompareInfo.getUserMeasureScoreMap()));
		if (Objects.nonNull(soundCompareInfo.getUserMeasureScoreMap()) && soundCompareInfo.getUserMeasureScoreMap().containsKey(-1)){
			Map<String, Object> finalScore = soundCompareInfo.getUserMeasureScoreMap().get(-1);
			sysMusicCompareRecord.setScore((BigDecimal) finalScore.get("score"));
			sysMusicCompareRecord.setIntonation((BigDecimal) finalScore.get("intonation"));
			sysMusicCompareRecord.setCadence((BigDecimal) finalScore.get("cadence"));
			sysMusicCompareRecord.setIntegrity((BigDecimal) finalScore.get("integrity"));
		}
		sysMusicCompareRecord.setRecordFilePath(soundCompareInfo.getRecordFilePath());
		sysMusicCompareRecord.setDeviceType(soundCompareInfo.getDeviceType());
		sysMusicCompareRecord.setClientId(soundCompareInfo.getClientId());
		if(Objects.nonNull(soundCompareInfo.getFile())){
			sysMusicCompareRecord.setPlayTime(soundCompareInfo.getFile().length()/(SoundCompareHandler.soundCompareConfig.audioFormat.getFrameSize()*SoundCompareHandler.soundCompareConfig.audioFormat.getFrameRate()));
		}
		sysMusicCompareRecord.setFeature(FeatureType.CLOUD_STUDY_EVALUATION);
		sysMusicCompareRecordDao.insert(sysMusicCompareRecord);
		studentDao.addStudentCloudStudySequenceDays(user.getId());
	}

	@Override
	public StatDto rankingList(Integer headUserId, String startDate, String endDate) {
		SysUser user = teacherDao.getUser(headUserId);
		StatDto result = new StatDto();
		MusicCompareRankingDto head = new MusicCompareRankingDto();
		head.setUserId(headUserId);
		List<MusicCompareRankingDto> userTrainStat = sysMusicCompareRecordDao.getUserTrainStat(startDate, endDate);
		List<MusicCompareRankingDto> detail = new ArrayList<>();
		userTrainStat.sort(Comparator.comparing(m->m.getTrainDays()+m.getTrainNum()+m.getTrainTime()));
		for (int i = 0; i < userTrainStat.size(); i++) {
			userTrainStat.get(i).setRankNum(userTrainStat.size()-i);
			userTrainStat.get(i).setTrainTime(userTrainStat.get(i).getTrainTime()/60);
			if(userTrainStat.get(i).getUserId().equals(headUserId)){
				head = userTrainStat.get(i);
			}
			if(userTrainStat.get(i).getRankNum()<=10){
				detail.add(userTrainStat.get(i));
			}
		}
		if(Objects.isNull(head.getRankNum())&&Objects.nonNull(user)){
			head.setAvatar(user.getAvatar());
			head.setStudentName(user.getUsername());
		}
		result.setHead(head);
		detail.sort(Comparator.comparing(MusicCompareRankingDto::getRankNum).thenComparing(MusicCompareRankingDto::getUserId));
		result.setDetail(detail);
		return result;
	}

	@Override
	public Map<String, Object> studentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		StudentTrainOverviewDto userTrainOverView = new StudentTrainOverviewDto();

		List<StudentTrainChartDto> userTrainChartData = new ArrayList<>();
		if(queryInfo.getPage()==1){
			userTrainOverView = sysMusicCompareRecordDao.getUserTrainOverView(queryInfo.getUserId());
			userTrainChartData = sysMusicCompareRecordDao.getUserTrainChartData(queryInfo.getUserId(), queryInfo.getStartTime(), queryInfo.getEndTime());
		}

		if(CollectionUtils.isEmpty(userTrainChartData)||userTrainChartData.size()!=7){
			Set<LocalDate> trainDates = userTrainChartData.stream().map(StudentTrainChartDto::getTrainDate).collect(Collectors.toSet());
			LocalDate startLocalDate = LocalDate.parse(queryInfo.getStartTime(), DateUtil.dateFormatter);
			for (int i = 1; i <= 7; i++) {
				if(!trainDates.contains(startLocalDate)){
					userTrainChartData.add(new StudentTrainChartDto(0, 0, startLocalDate));
				}
				startLocalDate=startLocalDate.plusDays(1);
			}
		}

		PageInfo<SysMusicCompareRecord> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<SysMusicCompareRecord> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		Map<String, Object> result = new HashMap<>();
		result.put("userTrainOverView",userTrainOverView);
		result.put("userTrainChartData",userTrainChartData);
		result.put("detail",pageInfo);
		return result;
	}

	@Override
	public PageInfo<MusicCompareRankingDto> queryMusicGroupStudentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<MusicCompareRankingDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<MusicCompareRankingDto> dataList = new ArrayList<>();
		int count = sysMusicCompareRecordDao.countMusicGroupStudentTrainData(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList =sysMusicCompareRecordDao.queryMusicGroupStudentTrainData(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<SysMusicCompareRecord> queryStudentTrainData(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<SysMusicCompareRecord> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<SysMusicCompareRecord> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
    public PageInfo<CountStudentTrainDataDto> countStudentTrain(SysMusicCompareRecordQueryInfo queryInfo) {
		PageInfo<CountStudentTrainDataDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<CountStudentTrainDataDto> dataList = new ArrayList<>();
		int count = sysMusicCompareRecordDao.countStudentTrain(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList =sysMusicCompareRecordDao.queryStudentTrain(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
    }
}
